@@ -66,10 +66,10 @@ from hypothesis.errors import (
     Unsatisfiable,
     UnsatisfiedAssumption,
 )
-from hypothesis.executors import default_new_style_executor, new_style_executor
 from hypothesis.internal.compat import (
     PYPY,
     BaseExceptionGroup,
+    add_note,
     bad_django_TestCase,
     get_type_hints,
     int_from_bytes,
@@ -80,13 +80,18 @@ from hypothesis.internal.conjecture.junkdrawer import ensure_free_stackframes
 from hypothesis.internal.conjecture.shrinker import sort_key
 from hypothesis.internal.entropy import deterministic_PRNG
 from hypothesis.internal.escalation import (
+    InterestingOrigin,
     current_pytest_item,
     escalate_hypothesis_internal_error,
     format_exception,
-    get_interesting_origin,
     get_trimmed_traceback,
 )
 from hypothesis.internal.healthcheck import fail_health_check
+from hypothesis.internal.observability import (
+    TESTCASE_CALLBACKS,
+    deliver_json_blob,
+    make_testcase,
+)
 from hypothesis.internal.reflection import (
     convert_positional_arguments,
     define_function_signature,
@@ -99,7 +104,12 @@ from hypothesis.internal.reflection import (
     proxies,
     repr_call,
 )
-from hypothesis.internal.scrutineer import Tracer, explanatory_lines
+from hypothesis.internal.scrutineer import (
+    Trace,
+    Tracer,
+    explanatory_lines,
+    tractable_coverage_report,
+)
 from hypothesis.internal.validation import check_type
 from hypothesis.reporting import (
     current_verbosity,
@@ -107,13 +117,14 @@ from hypothesis.reporting import (
     verbose_report,
     with_reporter,
 )
-from hypothesis.statistics import describe_targets, note_statistics
+from hypothesis.statistics import describe_statistics, describe_targets, note_statistics
 from hypothesis.strategies._internal.misc import NOTHING
 from hypothesis.strategies._internal.strategies import (
     Ex,
     SearchStrategy,
     check_strategy,
 )
+from hypothesis.strategies._internal.utils import to_jsonable
 from hypothesis.vendor.pretty import RepresentationPrinter
 from hypothesis.version import __version__
 
@@ -165,14 +176,15 @@ class example:
 
     def xfail(
         self,
-        condition: bool = True,
+        condition: bool = True,  # noqa: FBT002
         *,
         reason: str = "",
         raises: Union[
             Type[BaseException], Tuple[Type[BaseException], ...]
         ] = BaseException,
     ) -> "example":
-        """Mark this example as an expected failure, like pytest.mark.xfail().
+        """Mark this example as an expected failure, similarly to
+        :obj:`pytest.mark.xfail(strict=True) <pytest.mark.xfail>`.
 
         Expected-failing examples allow you to check that your test does fail on
         some examples, and therefore build confidence that *passing* tests are
@@ -187,6 +199,37 @@ class example:
             @example(...).xfail(condition=sys.platform != "linux", raises=OSError)
             def test(x):
                 pass
+
+        .. note::
+
+            Expected-failing examples are handled separately from those generated
+            by strategies, so you should usually ensure that there is no overlap.
+
+            .. code-block:: python
+
+                @example(x=1, y=0).xfail(raises=ZeroDivisionError)
+                @given(x=st.just(1), y=st.integers())  # Missing `.filter(bool)`!
+                def test_fraction(x, y):
+                    # This test will try the explicit example and see it fail as
+                    # expected, then go on to generate more examples from the
+                    # strategy.  If we happen to generate y=0, the test will fail
+                    # because only the explicit example is treated as xfailing.
+                    x / y
+
+        Note that this "method chaining" syntax requires Python 3.9 or later, for
+        :pep:`614` relaxing grammar restrictions on decorators.  If you need to
+        support older versions of Python, you can use an identity function:
+
+        .. code-block:: python
+
+            def identity(x):
+                return x
+
+
+            @identity(example(...).xfail())
+            def test(x):
+                pass
+
         """
         check_type(bool, condition, "condition")
         check_type(str, reason, "reason")
@@ -452,13 +495,14 @@ def execute_explicit_examples(state, wrapped_test, arguments, kwargs, original_s
 
         with local_settings(state.settings):
             fragments_reported = []
+            empty_data = ConjectureData.for_buffer(b"")
             try:
                 bits = ", ".join(nicerepr(x) for x in arguments) + ", ".join(
                     f"{k}={nicerepr(v)}" for k, v in example_kwargs.items()
                 )
                 execute_example = partial(
                     state.execute_once,
-                    ConjectureData.for_buffer(b""),
+                    empty_data,
                     is_final=True,
                     print_example=True,
                     example_kwargs=example_kwargs,
@@ -468,11 +512,17 @@ def execute_explicit_examples(state, wrapped_test, arguments, kwargs, original_s
                         execute_example()
                     else:
                         # @example(...).xfail(...)
+
                         try:
                             execute_example()
                         except failure_exceptions_to_catch() as err:
                             if not isinstance(err, example.raises):
                                 raise
+                            # Save a string form of this example; we'll warn if it's
+                            # ever generated by the strategy (which can't be xfailed)
+                            state.xfail_example_reprs.add(
+                                repr_call(state.test, arguments, example_kwargs)
+                            )
                         except example.raises as err:
                             # We'd usually check this as early as possible, but it's
                             # possible for failure_exceptions_to_catch() to grow when
@@ -506,7 +556,8 @@ def execute_explicit_examples(state, wrapped_test, arguments, kwargs, original_s
                 # development, this is rather useful to replay Hypothesis' part of
                 # a saved failure when other arguments are supplied by e.g. pytest.
                 # See https://github.com/HypothesisWorks/hypothesis/issues/2125
-                pass
+                with contextlib.suppress(StopTest):
+                    empty_data.conclude_test(Status.INVALID)
             except BaseException as err:
                 # In order to support reporting of multiple failing examples, we yield
                 # each of the (report text, error) pairs we find back to the top-level
@@ -529,6 +580,8 @@ def execute_explicit_examples(state, wrapped_test, arguments, kwargs, original_s
                     new.__cause__ = err
                     err = new
 
+                with contextlib.suppress(StopTest):
+                    empty_data.conclude_test(Status.INVALID)
                 yield (fragments_reported, err)
                 if (
                     state.settings.report_multiple_bugs
@@ -545,6 +598,15 @@ def execute_explicit_examples(state, wrapped_test, arguments, kwargs, original_s
                         "Falsifying example", "Falsifying explicit example", 1
                     )
 
+                tc = make_testcase(
+                    start_timestamp=state._start_timestamp,
+                    test_name_or_nodeid=state.test_identifier,
+                    data=empty_data,
+                    how_generated="explicit example",
+                    string_repr=state._string_repr,
+                )
+                deliver_json_blob(tc)
+
             if fragments_reported:
                 verbose_report(fragments_reported[0].replace("Falsifying", "Trying", 1))
                 for f in fragments_reported[1:]:
@@ -572,10 +634,10 @@ def get_random_for_wrapped_test(test, wrapped_test):
 
 @attr.s
 class Stuff:
-    selfy = attr.ib(default=None)
-    args = attr.ib(factory=tuple)
-    kwargs = attr.ib(factory=dict)
-    given_kwargs = attr.ib(factory=dict)
+    selfy: Any = attr.ib(default=None)
+    args: tuple = attr.ib(factory=tuple)
+    kwargs: dict = attr.ib(factory=dict)
+    given_kwargs: dict = attr.ib(factory=dict)
 
 
 def process_arguments_to_given(wrapped_test, arguments, kwargs, given_kwargs, params):
@@ -596,8 +658,6 @@ def process_arguments_to_given(wrapped_test, arguments, kwargs, given_kwargs, pa
     if is_mock(selfy):
         selfy = None
 
-    test_runner = new_style_executor(selfy)
-
     arguments = tuple(arguments)
 
     with ensure_free_stackframes():
@@ -607,7 +667,7 @@ def process_arguments_to_given(wrapped_test, arguments, kwargs, given_kwargs, pa
 
     stuff = Stuff(selfy=selfy, args=arguments, kwargs=kwargs, given_kwargs=given_kwargs)
 
-    return arguments, kwargs, test_runner, stuff
+    return arguments, kwargs, stuff
 
 
 def skip_exceptions_to_reraise():
@@ -665,19 +725,48 @@ def new_given_signature(original_sig, given_kwargs):
     )
 
 
+def default_executor(data, function):
+    return function(data)
+
+
+def get_executor(runner):
+    try:
+        execute_example = runner.execute_example
+    except AttributeError:
+        pass
+    else:
+        return lambda data, function: execute_example(partial(function, data))
+
+    if hasattr(runner, "setup_example") or hasattr(runner, "teardown_example"):
+        setup = getattr(runner, "setup_example", None) or (lambda: None)
+        teardown = getattr(runner, "teardown_example", None) or (lambda ex: None)
+
+        def execute(data, function):
+            token = None
+            try:
+                token = setup()
+                return function(data)
+            finally:
+                teardown(token)
+
+        return execute
+
+    return default_executor
+
+
 class StateForActualGivenExecution:
-    def __init__(self, test_runner, stuff, test, settings, random, wrapped_test):
-        self.test_runner = test_runner
+    def __init__(self, stuff, test, settings, random, wrapped_test):
+        self.test_runner = get_executor(stuff.selfy)
         self.stuff = stuff
         self.settings = settings
         self.last_exception = None
         self.falsifying_examples = ()
         self.random = random
-        self.__test_runtime = None
         self.ever_executed = False
 
         self.is_find = getattr(wrapped_test, "_hypothesis_internal_is_find", False)
         self.wrapped_test = wrapped_test
+        self.xfail_example_reprs = set()
 
         self.test = test
 
@@ -690,10 +779,21 @@ class StateForActualGivenExecution:
         self.failed_due_to_deadline = False
 
         self.explain_traces = defaultdict(set)
+        self._start_timestamp = time.time()
+        self._string_repr = ""
+        self._jsonable_arguments = {}
+        self._timing_features = {}
+
+    @property
+    def test_identifier(self):
+        return getattr(
+            current_pytest_item.value, "nodeid", None
+        ) or get_pretty_function_description(self.wrapped_test)
 
     def execute_once(
         self,
         data,
+        *,
         print_example=False,
         is_final=False,
         expected_failure=None,
@@ -713,6 +813,7 @@ class StateForActualGivenExecution:
         self.ever_executed = True
         data.is_find = self.is_find
 
+        self._string_repr = ""
         text_repr = None
         if self.settings.deadline is None:
             test = self.test
@@ -720,16 +821,23 @@ class StateForActualGivenExecution:
 
             @proxies(self.test)
             def test(*args, **kwargs):
-                self.__test_runtime = None
+                arg_drawtime = sum(data.draw_times)
                 initial_draws = len(data.draw_times)
                 start = time.perf_counter()
-                result = self.test(*args, **kwargs)
-                finish = time.perf_counter()
-                internal_draw_time = sum(data.draw_times[initial_draws:])
-                runtime = datetime.timedelta(
-                    seconds=finish - start - internal_draw_time
-                )
-                self.__test_runtime = runtime
+                try:
+                    result = self.test(*args, **kwargs)
+                finally:
+                    finish = time.perf_counter()
+                    internal_draw_time = sum(data.draw_times[initial_draws:])
+                    runtime = datetime.timedelta(
+                        seconds=finish - start - internal_draw_time
+                    )
+                    self._timing_features = {
+                        "time_running_test": finish - start - internal_draw_time,
+                        "time_drawing_args": arg_drawtime,
+                        "time_interactive_draws": internal_draw_time,
+                    }
+
                 current_deadline = self.settings.deadline
                 if not is_final:
                     current_deadline = (current_deadline // 4) * 5
@@ -739,62 +847,92 @@ class StateForActualGivenExecution:
 
         def run(data):
             # Set up dynamic context needed by a single test run.
-            with local_settings(self.settings):
-                with deterministic_PRNG():
-                    with BuildContext(data, is_final=is_final) as context:
-                        if self.stuff.selfy is not None:
-                            data.hypothesis_runner = self.stuff.selfy
-                        # Generate all arguments to the test function.
-                        args = self.stuff.args
-                        kwargs = dict(self.stuff.kwargs)
-                        if example_kwargs is None:
-                            a, kw, argslices = context.prep_args_kwargs_from_strategies(
-                                (), self.stuff.given_kwargs
-                            )
-                            assert not a, "strategies all moved to kwargs by now"
-                        else:
-                            kw = example_kwargs
-                            argslices = {}
-                        kwargs.update(kw)
-                        if expected_failure is not None:
-                            nonlocal text_repr
-                            text_repr = repr_call(test, args, kwargs)
+            if self.stuff.selfy is not None:
+                data.hypothesis_runner = self.stuff.selfy
+            # Generate all arguments to the test function.
+            args = self.stuff.args
+            kwargs = dict(self.stuff.kwargs)
+            if example_kwargs is None:
+                a, kw, argslices = context.prep_args_kwargs_from_strategies(
+                    (), self.stuff.given_kwargs
+                )
+                assert not a, "strategies all moved to kwargs by now"
+            else:
+                kw = example_kwargs
+                argslices = {}
+            kwargs.update(kw)
+            if expected_failure is not None:
+                nonlocal text_repr
+                text_repr = repr_call(test, args, kwargs)
+                if text_repr in self.xfail_example_reprs:
+                    warnings.warn(
+                        f"We generated {text_repr}, which seems identical "
+                        "to one of your `@example(...).xfail()` cases.  "
+                        "Revise the strategy to avoid this overlap?",
+                        HypothesisWarning,
+                        # Checked in test_generating_xfailed_examples_warns!
+                        stacklevel=6,
+                    )
 
-                        if print_example or current_verbosity() >= Verbosity.verbose:
-                            printer = RepresentationPrinter(context=context)
-                            if print_example:
-                                printer.text("Falsifying example:")
-                            else:
-                                printer.text("Trying example:")
+            if print_example or current_verbosity() >= Verbosity.verbose:
+                printer = RepresentationPrinter(context=context)
+                if print_example:
+                    printer.text("Falsifying example:")
+                else:
+                    printer.text("Trying example:")
 
-                            if self.print_given_args:
-                                printer.text(" ")
-                                printer.repr_call(
-                                    test.__name__,
-                                    args,
-                                    kwargs,
-                                    force_split=True,
-                                    arg_slices=argslices,
-                                    leading_comment=(
-                                        "# " + context.data.slice_comments[(0, 0)]
-                                        if (0, 0) in context.data.slice_comments
-                                        else None
-                                    ),
-                                )
-                            report(printer.getvalue())
-                        return test(*args, **kwargs)
+                if self.print_given_args:
+                    printer.text(" ")
+                    printer.repr_call(
+                        test.__name__,
+                        args,
+                        kwargs,
+                        force_split=True,
+                        arg_slices=argslices,
+                        leading_comment=(
+                            "# " + context.data.slice_comments[(0, 0)]
+                            if (0, 0) in context.data.slice_comments
+                            else None
+                        ),
+                    )
+                report(printer.getvalue())
 
-        # Run the test function once, via the executor hook.
-        # In most cases this will delegate straight to `run(data)`.
-        result = self.test_runner(data, run)
+            if TESTCASE_CALLBACKS:
+                printer = RepresentationPrinter(context=context)
+                printer.repr_call(
+                    test.__name__,
+                    args,
+                    kwargs,
+                    force_split=True,
+                    arg_slices=argslices,
+                    leading_comment=(
+                        "# " + context.data.slice_comments[(0, 0)]
+                        if (0, 0) in context.data.slice_comments
+                        else None
+                    ),
+                )
+                self._string_repr = printer.getvalue()
+                self._jsonable_arguments = {
+                    **dict(enumerate(map(to_jsonable, args))),
+                    **{k: to_jsonable(v) for k, v in kwargs.items()},
+                }
+            return test(*args, **kwargs)
+
+        # self.test_runner can include the execute_example method, or setup/teardown
+        # _example, so it's important to get the PRNG and build context in place first.
+        with local_settings(self.settings):
+            with deterministic_PRNG():
+                with BuildContext(data, is_final=is_final) as context:
+                    # Run the test function once, via the executor hook.
+                    # In most cases this will delegate straight to `run(data)`.
+                    result = self.test_runner(data, run)
 
         # If a failure was expected, it should have been raised already, so
         # instead raise an appropriate diagnostic error.
         if expected_failure is not None:
             exception, traceback = expected_failure
-            if (
-                isinstance(exception, DeadlineExceeded)
-                and self.__test_runtime is not None
+            if isinstance(exception, DeadlineExceeded) and (
+                runtime_secs := self._timing_features.get("time_running_test")
             ):
                 report(
                     "Unreliable test timings! On an initial run, this "
@@ -806,7 +944,7 @@ class StateForActualGivenExecution:
                     % (
                         exception.runtime.total_seconds() * 1000,
                         self.settings.deadline.total_seconds() * 1000,
-                        self.__test_runtime.total_seconds() * 1000,
+                        runtime_secs * 1000,
                     )
                 )
             else:
@@ -817,7 +955,7 @@ class StateForActualGivenExecution:
             ) from exception
         return result
 
-    def _execute_once_for_engine(self, data):
+    def _execute_once_for_engine(self, data: ConjectureData) -> None:
         """Wrapper around ``execute_once`` that intercepts test failure
         exceptions and single-test control exceptions, and turns them into
         appropriate method calls to `data` instead.
@@ -825,27 +963,27 @@ class StateForActualGivenExecution:
         This allows the engine to assume that any exception other than
         ``StopTest`` must be a fatal error, and should stop the entire engine.
         """
+        trace: Trace = set()
         try:
-            trace = frozenset()
-            if (
+            _can_trace = (
+                sys.gettrace() is None or sys.version_info[:2] >= (3, 12)
+            ) and not PYPY
+            _trace_obs = TESTCASE_CALLBACKS
+            _trace_failure = (
                 self.failed_normally
                 and not self.failed_due_to_deadline
-                and Phase.shrink in self.settings.phases
-                and Phase.explain in self.settings.phases
-                and sys.gettrace() is None
-                and not PYPY
-            ):  # pragma: no cover
+                and {Phase.shrink, Phase.explain}.issubset(self.settings.phases)
+            )
+            if _can_trace and (_trace_obs or _trace_failure):  # pragma: no cover
                 # This is in fact covered by our *non-coverage* tests, but due to the
                 # settrace() contention *not* by our coverage tests.  Ah well.
-                tracer = Tracer()
-                try:
-                    sys.settrace(tracer.trace)
-                    result = self.execute_once(data)
-                    if data.status == Status.VALID:
-                        self.explain_traces[None].add(frozenset(tracer.branches))
-                finally:
-                    sys.settrace(None)
-                    trace = frozenset(tracer.branches)
+                with Tracer() as tracer:
+                    try:
+                        result = self.execute_once(data)
+                        if data.status == Status.VALID:
+                            self.explain_traces[None].add(frozenset(tracer.branches))
+                    finally:
+                        trace = tracer.branches
             else:
                 result = self.execute_once(data)
             if result is not None:
@@ -866,7 +1004,8 @@ class StateForActualGivenExecution:
         except (
             HypothesisDeprecationWarning,
             FailedHealthCheck,
-        ) + skip_exceptions_to_reraise():
+            *skip_exceptions_to_reraise(),
+        ):
             # These are fatal errors or control exceptions that should stop the
             # engine, so we re-raise them.
             raise
@@ -887,20 +1026,40 @@ class StateForActualGivenExecution:
 
                 tb = get_trimmed_traceback()
                 info = data.extra_information
-                info.__expected_traceback = format_exception(e, tb)
-                info.__expected_exception = e
-                verbose_report(info.__expected_traceback)
+                info._expected_traceback = format_exception(e, tb)  # type: ignore
+                info._expected_exception = e  # type: ignore
+                verbose_report(info._expected_traceback)  # type: ignore
 
                 self.failed_normally = True
 
-                interesting_origin = get_interesting_origin(e)
+                interesting_origin = InterestingOrigin.from_exception(e)
                 if trace:  # pragma: no cover
                     # Trace collection is explicitly disabled under coverage.
-                    self.explain_traces[interesting_origin].add(trace)
+                    self.explain_traces[interesting_origin].add(frozenset(trace))
                 if interesting_origin[0] == DeadlineExceeded:
                     self.failed_due_to_deadline = True
                     self.explain_traces.clear()
-                data.mark_interesting(interesting_origin)
+                data.mark_interesting(interesting_origin)  # type: ignore  # mypy bug?
+        finally:
+            # Conditional here so we can save some time constructing the payload; in
+            # other cases (without coverage) it's cheap enough to do that regardless.
+            if TESTCASE_CALLBACKS:
+                if self.failed_normally or self.failed_due_to_deadline:
+                    phase = "shrink"
+                else:
+                    phase = "unknown"
+                tc = make_testcase(
+                    start_timestamp=self._start_timestamp,
+                    test_name_or_nodeid=self.test_identifier,
+                    data=data,
+                    how_generated=f"generated during {phase} phase",
+                    string_repr=self._string_repr,
+                    arguments={**self._jsonable_arguments, **data._observability_args},
+                    metadata=self._timing_features,
+                    coverage=tractable_coverage_report(trace) or None,
+                )
+                deliver_json_blob(tc)
+            self._timing_features.clear()
 
     def run_engine(self):
         """Run the test function many times, on database input and generated
@@ -926,6 +1085,15 @@ class StateForActualGivenExecution:
         # on different inputs.
         runner.run()
         note_statistics(runner.statistics)
+        deliver_json_blob(
+            {
+                "type": "info",
+                "run_start": self._start_timestamp,
+                "property": self.test_identifier,
+                "title": "Hypothesis Statistics",
+                "content": describe_statistics(runner.statistics),
+            }
+        )
 
         if runner.call_count == 0:
             return
@@ -960,9 +1128,13 @@ class StateForActualGivenExecution:
             info = falsifying_example.extra_information
             fragments = []
 
-            ran_example = ConjectureData.for_buffer(falsifying_example.buffer)
+            ran_example = runner.new_conjecture_data_for_buffer(
+                falsifying_example.buffer
+            )
             ran_example.slice_comments = falsifying_example.slice_comments
-            assert info.__expected_exception is not None
+            tb = None
+            origin = None
+            assert info._expected_exception is not None
             try:
                 with with_reporter(fragments.append):
                     self.execute_once(
@@ -970,8 +1142,8 @@ class StateForActualGivenExecution:
                         print_example=not self.is_find,
                         is_final=True,
                         expected_failure=(
-                            info.__expected_exception,
-                            info.__expected_traceback,
+                            info._expected_exception,
+                            info._expected_traceback,
                         ),
                     )
             except (UnsatisfiedAssumption, StopTest) as e:
@@ -987,10 +1159,34 @@ class StateForActualGivenExecution:
                 errors_to_report.append(
                     (fragments, e.with_traceback(get_trimmed_traceback()))
                 )
+                tb = format_exception(e, get_trimmed_traceback(e))
+                origin = InterestingOrigin.from_exception(e)
             else:
                 # execute_once() will always raise either the expected error, or Flaky.
                 raise NotImplementedError("This should be unreachable")
             finally:
+                # log our observability line for the final failing example
+                tc = {
+                    "type": "test_case",
+                    "run_start": self._start_timestamp,
+                    "property": self.test_identifier,
+                    "status": "passed" if sys.exc_info()[0] else "failed",
+                    "status_reason": str(origin or "unexpected/flaky pass"),
+                    "representation": self._string_repr,
+                    "how_generated": "minimal failing example",
+                    "features": {
+                        **{
+                            k: v
+                            for k, v in ran_example.target_observations.items()
+                            if isinstance(k, str)
+                        },
+                        **ran_example.events,
+                        **self._timing_features,
+                    },
+                    "coverage": None,  # TODO: expose this?
+                    "metadata": {"traceback": tb},
+                }
+                deliver_json_blob(tc)
                 # Whether or not replay actually raised the exception again, we want
                 # to print the reproduce_failure decorator for the failing example.
                 if self.settings.print_blob:
@@ -1007,15 +1203,6 @@ class StateForActualGivenExecution:
         _raise_to_user(errors_to_report, self.settings, report_lines)
 
 
-def add_note(exc, note):
-    try:
-        exc.add_note(note)
-    except AttributeError:
-        if not hasattr(exc, "__notes__"):
-            exc.__notes__ = []
-        exc.__notes__.append(note)
-
-
 def _raise_to_user(errors_to_report, settings, target_lines, trailer=""):
     """Helper function for attaching notes and grouping multiple errors."""
     failing_prefix = "Falsifying example: "
@@ -1104,7 +1291,16 @@ class HypothesisHandle:
 
 @overload
 def given(
-    *_given_arguments: Union[SearchStrategy[Any], EllipsisType],
+    _: EllipsisType, /
+) -> Callable[
+    [Callable[..., Optional[Coroutine[Any, Any, None]]]], Callable[[], None]
+]:  # pragma: no cover
+    ...
+
+
+@overload
+def given(
+    *_given_arguments: SearchStrategy[Any],
 ) -> Callable[
     [Callable[..., Optional[Coroutine[Any, Any, None]]]], Callable[..., None]
 ]:  # pragma: no cover
@@ -1187,6 +1383,8 @@ def given(
                 )
             given_kwargs[name] = st.from_type(hints[name])
 
+        prev_self = Unset = object()
+
         @impersonate(test)
         @define_function_signature(test.__name__, test.__doc__, new_signature)
         def wrapped_test(*arguments, **kwargs):
@@ -1209,14 +1407,13 @@ def given(
 
             random = get_random_for_wrapped_test(test, wrapped_test)
 
-            processed_args = process_arguments_to_given(
+            arguments, kwargs, stuff = process_arguments_to_given(
                 wrapped_test, arguments, kwargs, given_kwargs, new_signature.parameters
             )
-            arguments, kwargs, test_runner, stuff = processed_args
 
             if (
                 inspect.iscoroutinefunction(test)
-                and test_runner is default_new_style_executor
+                and get_executor(stuff.selfy) is default_executor
             ):
                 # See https://github.com/HypothesisWorks/hypothesis/issues/3054
                 # If our custom executor doesn't handle coroutines, or we return an
@@ -1248,9 +1445,26 @@ def given(
                     "to ensure that each example is run in a separate "
                     "database transaction."
                 )
+            if settings.database is not None:
+                nonlocal prev_self
+                # Check selfy really is self (not e.g. a mock) before we health-check
+                cur_self = (
+                    stuff.selfy
+                    if getattr(type(stuff.selfy), test.__name__, None) is wrapped_test
+                    else None
+                )
+                if prev_self is Unset:
+                    prev_self = cur_self
+                elif cur_self is not prev_self:
+                    msg = (
+                        f"The method {test.__qualname__} was called from multiple "
+                        "different executors. This may lead to flaky tests and "
+                        "nonreproducible errors when replaying from database."
+                    )
+                    fail_health_check(settings, msg, HealthCheck.differing_executors)
 
             state = StateForActualGivenExecution(
-                test_runner, stuff, test, settings, random, wrapped_test
+                stuff, test, settings, random, wrapped_test
             )
 
             reproduce_failure = wrapped_test._hypothesis_internal_use_reproduce_failure
@@ -1393,13 +1607,13 @@ def given(
                 parent=wrapped_test._hypothesis_internal_use_settings, deadline=None
             )
             random = get_random_for_wrapped_test(test, wrapped_test)
-            _args, _kwargs, test_runner, stuff = process_arguments_to_given(
+            _args, _kwargs, stuff = process_arguments_to_given(
                 wrapped_test, (), {}, given_kwargs, new_signature.parameters
             )
             assert not _args
             assert not _kwargs
             state = StateForActualGivenExecution(
-                test_runner, stuff, test, settings, random, wrapped_test
+                stuff, test, settings, random, wrapped_test
             )
             digest = function_digest(test)
             # We track the minimal-so-far example for each distinct origin, so
@@ -1477,6 +1691,9 @@ def find(
     )
 
     if database_key is None and settings.database is not None:
+        # Note: The database key is not guaranteed to be unique. If not, replaying
+        # of database examples may fail to reproduce due to being replayed on the
+        # wrong condition.
         database_key = function_digest(condition)
 
     if not isinstance(specifier, SearchStrategy):
@@ -1493,7 +1710,7 @@ def find(
     def test(v):
         if condition(v):
             last[:] = [v]
-            raise Found()
+            raise Found
 
     if random is not None:
         test = seed(random.getrandbits(64))(test)
