@@ -19,24 +19,24 @@ import inspect
 import io
 import ipaddress
 import numbers
+import operator
 import os
 import random
 import re
 import sys
 import typing
 import uuid
+import warnings
+from functools import partial
 from pathlib import PurePath
 from types import FunctionType
+from typing import TYPE_CHECKING, Any, Iterator, Tuple, get_args, get_origin
 
 from hypothesis import strategies as st
-from hypothesis.errors import InvalidArgument, ResolutionFailed
-from hypothesis.internal.compat import (
-    PYPY,
-    BaseExceptionGroup,
-    ExceptionGroup,
-    get_origin,
-)
+from hypothesis.errors import HypothesisWarning, InvalidArgument, ResolutionFailed
+from hypothesis.internal.compat import PYPY, BaseExceptionGroup, ExceptionGroup
 from hypothesis.internal.conjecture.utils import many as conjecture_utils_many
+from hypothesis.internal.filtering import max_len, min_len
 from hypothesis.strategies._internal.datetime import zoneinfo  # type: ignore
 from hypothesis.strategies._internal.ipaddress import (
     SPECIAL_IPv4_RANGES,
@@ -46,6 +46,9 @@ from hypothesis.strategies._internal.ipaddress import (
 from hypothesis.strategies._internal.lazy import unwrap_strategies
 from hypothesis.strategies._internal.strategies import OneOfStrategy
 
+if TYPE_CHECKING:
+    import annotated_types as at
+
 GenericAlias: typing.Any
 UnionType: typing.Any
 try:
@@ -84,11 +87,7 @@ try:
 except AttributeError:  # pragma: no cover
     pass  # `typing_extensions` might not be installed
 
-FinalTypes: tuple = ()
-try:
-    FinalTypes += (typing.Final,)
-except AttributeError:  # pragma: no cover
-    pass  # Is missing for `python<3.8`
+FinalTypes: tuple = (typing.Final,)
 try:
     FinalTypes += (typing_extensions.Final,)
 except AttributeError:  # pragma: no cover
@@ -98,7 +97,7 @@ ConcatenateTypes: tuple = ()
 try:
     ConcatenateTypes += (typing.Concatenate,)
 except AttributeError:  # pragma: no cover
-    pass  # Is missing for `python<3.8`
+    pass  # Is missing for `python<3.10`
 try:
     ConcatenateTypes += (typing_extensions.Concatenate,)
 except AttributeError:  # pragma: no cover
@@ -108,7 +107,7 @@ ParamSpecTypes: tuple = ()
 try:
     ParamSpecTypes += (typing.ParamSpec,)
 except AttributeError:  # pragma: no cover
-    pass  # Is missing for `python<3.8`
+    pass  # Is missing for `python<3.10`
 try:
     ParamSpecTypes += (typing_extensions.ParamSpec,)
 except AttributeError:  # pragma: no cover
@@ -118,7 +117,7 @@ TypeGuardTypes: tuple = ()
 try:
     TypeGuardTypes += (typing.TypeGuard,)
 except AttributeError:  # pragma: no cover
-    pass  # Is missing for `python<3.8`
+    pass  # Is missing for `python<3.10`
 try:
     TypeGuardTypes += (typing_extensions.TypeGuard,)
 except AttributeError:  # pragma: no cover
@@ -153,8 +152,7 @@ typing_root_type = (typing._Final, typing._GenericAlias)  # type: ignore
 # We use this to disallow all non-runtime types from being registered and resolved.
 # By "non-runtime" we mean: types that do not really exist in python's
 # and are just added for more fancy type annotations.
-# `Final` is a great example: it just indicates
-# that this value can't be reassigned.
+# `Final` is a great example: it just indicates that this value can't be reassigned.
 NON_RUNTIME_TYPES = (
     typing.Any,
     *ClassVarTypes,
@@ -187,15 +185,15 @@ for name in (
 
 def type_sorting_key(t):
     """Minimise to None, then non-container types, then container types."""
-    if not (is_a_type(t) or is_typing_literal(t)):  # This branch is for Python < 3.8
-        raise InvalidArgument(f"thing={t} must be a type")  # pragma: no cover
-    if t is None or t is type(None):  # noqa: E721
+    if t is None or t is type(None):
         return (-1, repr(t))
-    t = getattr(t, "__origin__", t)
-    if not isinstance(t, type):  # pragma: no cover
-        # Some generics in the typing module are not actually types in 3.7
-        return (2, repr(t))
-    return (int(issubclass(t, collections.abc.Container)), repr(t))
+    t = get_origin(t) or t
+    try:
+        is_container = int(issubclass(t, collections.abc.Container))
+    except Exception:  # pragma: no cover
+        # e.g. `typing_extensions.Literal` is not a container
+        is_container = 0
+    return (is_container, repr(t))
 
 
 def _compatible_args(args, superclass_args):
@@ -216,11 +214,8 @@ def try_issubclass(thing, superclass):
     try:
         # In this case we're looking at two distinct classes - which might be generics.
         # That brings in some complications:
-        if issubclass(
-            getattr(thing, "__origin__", None) or thing,
-            getattr(superclass, "__origin__", None) or superclass,
-        ):
-            superclass_args = getattr(superclass, "__args__", None)
+        if issubclass(get_origin(thing) or thing, get_origin(superclass) or superclass):
+            superclass_args = get_args(superclass)
             if not superclass_args:
                 # The superclass is not generic, so we're definitely a subclass.
                 return True
@@ -257,10 +252,7 @@ def is_a_new_type(thing):
 
 def is_a_union(thing):
     """Return True if thing is a typing.Union or types.UnionType (in py310)."""
-    return (
-        isinstance(thing, UnionType)
-        or getattr(thing, "__origin__", None) is typing.Union
-    )
+    return isinstance(thing, UnionType) or get_origin(thing) is typing.Union
 
 
 def is_a_type(thing):
@@ -269,11 +261,9 @@ def is_a_type(thing):
 
 
 def is_typing_literal(thing):
-    return (
-        hasattr(typing, "Literal")
-        and getattr(thing, "__origin__", None) == typing.Literal
-        or hasattr(typing_extensions, "Literal")
-        and getattr(thing, "__origin__", None) == typing_extensions.Literal
+    return get_origin(thing) in (
+        typing.Literal,
+        getattr(typing_extensions, "Literal", object()),
     )
 
 
@@ -284,19 +274,64 @@ def is_annotated_type(thing):
     )
 
 
-def find_annotated_strategy(annotated_type):  # pragma: no cover
-    flattened_meta = []
+def get_constraints_filter_map():
+    if at := sys.modules.get("annotated_types"):
+        return {
+            # Due to the order of operator.gt/ge/lt/le arguments, order is inversed:
+            at.Gt: lambda constraint: partial(operator.lt, constraint.gt),
+            at.Ge: lambda constraint: partial(operator.le, constraint.ge),
+            at.Lt: lambda constraint: partial(operator.gt, constraint.lt),
+            at.Le: lambda constraint: partial(operator.ge, constraint.le),
+            at.MinLen: lambda constraint: partial(min_len, constraint.min_length),
+            at.MaxLen: lambda constraint: partial(max_len, constraint.max_length),
+            at.Predicate: lambda constraint: constraint.func,
+        }
+    return {}  # pragma: no cover
 
-    all_args = (
-        *getattr(annotated_type, "__args__", ()),
-        *getattr(annotated_type, "__metadata__", ()),
-    )
-    for arg in all_args:
-        if is_annotated_type(arg):
-            flattened_meta.append(find_annotated_strategy(arg))
+
+def _get_constraints(args: Tuple[Any, ...]) -> Iterator["at.BaseMetadata"]:
+    if at := sys.modules.get("annotated_types"):
+        for arg in args:
+            if isinstance(arg, at.BaseMetadata):
+                yield arg
+            elif getattr(arg, "__is_annotated_types_grouped_metadata__", False):
+                yield from arg
+            elif isinstance(arg, slice) and arg.step in (1, None):
+                yield from at.Len(arg.start or 0, arg.stop)
+
+
+def find_annotated_strategy(annotated_type):
+    metadata = getattr(annotated_type, "__metadata__", ())
+
+    if any(is_annotated_type(arg) for arg in metadata):
+        # We are in the case where one of the metadata argument
+        # is itself an annotated type. Although supported at runtime,
+        # This shouldn't be allowed: we prefer to raise here
+        raise ResolutionFailed(
+            f"Failed to resolve strategy for the following Annotated type: {annotated_type}."
+            "Arguments to the Annotated type cannot be Annotated."
+        )
+    for arg in reversed(metadata):
         if isinstance(arg, st.SearchStrategy):
-            flattened_meta.append(arg)
-    return flattened_meta[-1] if flattened_meta else None
+            return arg
+
+    filter_conditions = []
+    if "annotated_types" in sys.modules:
+        unsupported = []
+        for constraint in _get_constraints(metadata):
+            if convert := get_constraints_filter_map().get(type(constraint)):
+                filter_conditions.append(convert(constraint))
+            else:
+                unsupported.append(constraint)
+        if unsupported:
+            msg = f"Ignoring unsupported {', '.join(map(repr, unsupported))}"
+            warnings.warn(msg, HypothesisWarning, stacklevel=2)
+
+    base_strategy = st.from_type(annotated_type.__origin__)
+    for filter_condition in filter_conditions:
+        base_strategy = base_strategy.filter(filter_condition)
+
+    return base_strategy
 
 
 def has_type_arguments(type_):
@@ -316,7 +351,7 @@ def is_generic_type(type_):
     # The ugly truth is that `MyClass`, `MyClass[T]`, and `MyClass[int]` are very different.
     # We check for `MyClass[T]` and `MyClass[int]` with the first condition,
     # while the second condition is for `MyClass`.
-    return isinstance(type_, typing_root_type + (GenericAlias,)) or (
+    return isinstance(type_, (*typing_root_type, GenericAlias)) or (
         isinstance(type_, type)
         and (typing.Generic in type_.__mro__ or hasattr(type_, "__class_getitem__"))
     )
@@ -341,28 +376,13 @@ def _try_import_forward_ref(thing, bound):  # pragma: no cover
 
 
 def from_typing_type(thing):
-    # We start with special-case support for Tuple, which isn't actually a generic
-    # type; then Final, Literal, and Annotated since they don't support `isinstance`.
+    # We start with Final, Literal, and Annotated since they don't support `isinstance`.
     #
     # We then explicitly error on non-Generic types, which don't carry enough
     # information to sensibly resolve to strategies at runtime.
     # Finally, we run a variation of the subclass lookup in `st.from_type`
     # among generic types in the lookup.
-    if getattr(thing, "__origin__", None) == tuple or isinstance(
-        thing, getattr(typing, "TupleMeta", ())
-    ):
-        elem_types = getattr(thing, "__tuple_params__", None) or ()
-        elem_types += getattr(thing, "__args__", None) or ()
-        if (
-            getattr(thing, "__tuple_use_ellipsis__", False)
-            or len(elem_types) == 2
-            and elem_types[-1] is Ellipsis
-        ):
-            return st.lists(st.from_type(elem_types[0])).map(tuple)
-        elif len(elem_types) == 1 and elem_types[0] == ():
-            return st.tuples()  # Empty tuple; see issue #1583
-        return st.tuples(*map(st.from_type, elem_types))
-    if hasattr(typing, "Final") and getattr(thing, "__origin__", None) == typing.Final:
+    if get_origin(thing) == typing.Final:
         return st.one_of([st.from_type(t) for t in thing.__args__])
     if is_typing_literal(thing):
         args_dfs_stack = list(thing.__args__)
@@ -391,8 +411,8 @@ def from_typing_type(thing):
         raise ResolutionFailed(f"Cannot resolve {thing} to a strategy")
 
     # Some "generic" classes are not generic *in* anything - for example both
-    # Hashable and Sized have `__args__ == ()` on Python 3.7 or later.
-    origin = getattr(thing, "__origin__", thing)
+    # Hashable and Sized have `__args__ == ()`
+    origin = get_origin(thing) or thing
     if (
         origin in vars(collections.abc).values()
         and len(getattr(thing, "__args__", None) or []) == 0
@@ -402,15 +422,26 @@ def from_typing_type(thing):
     # Parametrised generic types have their __origin__ attribute set to the
     # un-parametrised version, which we need to use in the subclass checks.
     # e.g.:     typing.List[int].__origin__ == typing.List
+    # (actually not sure if this is true since Python 3.9 or so)
     mapping = {
         k: v
         for k, v in _global_type_lookup.items()
         if is_generic_type(k) and try_issubclass(k, thing)
     }
-    # Drop some unusual cases for simplicity
-    for weird in (tuple, getattr(os, "_Environ", None)):
-        if len(mapping) > 1:
-            mapping.pop(weird, None)
+    # Drop some unusual cases for simplicity, including tuples or its
+    # subclasses (e.g. namedtuple)
+    if len(mapping) > 1:
+        _Environ = getattr(os, "_Environ", None)
+        mapping.pop(_Environ, None)
+    tuple_types = [
+        t
+        for t in mapping
+        if (isinstance(t, type) and issubclass(t, tuple)) or t is typing.Tuple
+    ]
+    if len(mapping) > len(tuple_types):
+        for tuple_type in tuple_types:
+            mapping.pop(tuple_type)
+
     # After we drop Python 3.8 and can rely on having generic builtin types, we'll
     # be able to simplify this logic by dropping the typing-module handling.
     if {dict, set, typing.Dict, typing.Set}.intersection(mapping):
@@ -423,6 +454,7 @@ def from_typing_type(thing):
         # the ghostwriter than it's worth, via undefined names in the repr.
         mapping.pop(collections.deque, None)
         mapping.pop(typing.Deque, None)
+
     if len(mapping) > 1:
         # issubclass treats bytestring as a kind of sequence, which it is,
         # but treating it as such breaks everything else when it is presumed
@@ -440,7 +472,7 @@ def from_typing_type(thing):
             union_elems = ()
         if not any(
             isinstance(T, type) and issubclass(int, get_origin(T) or T)
-            for T in list(union_elems) + [elem_type]
+            for T in [*union_elems, elem_type]
         ):
             mapping.pop(bytes, None)
             mapping.pop(collections.abc.ByteString, None)
@@ -460,9 +492,13 @@ def from_typing_type(thing):
             mapping.pop(t)
     # Sort strategies according to our type-sorting heuristic for stable output
     strategies = [
-        v if isinstance(v, st.SearchStrategy) else v(thing)
-        for k, v in sorted(mapping.items(), key=lambda kv: type_sorting_key(kv[0]))
-        if sum(try_issubclass(k, T) for T in mapping) == 1
+        s
+        for s in (
+            v if isinstance(v, st.SearchStrategy) else v(thing)
+            for k, v in sorted(mapping.items(), key=lambda kv: type_sorting_key(kv[0]))
+            if sum(try_issubclass(k, T) for T in mapping) == 1
+        )
+        if s != NotImplemented
     ]
     empty = ", ".join(repr(s) for s in strategies if s.is_empty)
     if empty or not strategies:
@@ -500,6 +536,14 @@ utc_offsets = st.builds(
 # As a general rule, we try to limit this to scalars because from_type()
 # would have to decide on arbitrary collection elements, and we'd rather
 # not (with typing module generic types and some builtins as exceptions).
+#
+# Strategy Callables may return NotImplemented, which should be treated in the
+# same way as if the type was not registered.
+#
+# Note that NotImplemented cannot be typed in Python 3.8 because there's no type
+# exposed for it, and NotImplemented itself is typed as Any so that it can be
+# returned without being listed in a function signature:
+# https://github.com/python/mypy/issues/6710#issuecomment-485580032
 _global_type_lookup: typing.Dict[
     type, typing.Union[st.SearchStrategy, typing.Callable[[type], st.SearchStrategy]]
 ] = {
@@ -608,7 +652,7 @@ if PYPY:
 
 
 _global_type_lookup[type] = st.sampled_from(
-    [type(None)] + sorted(_global_type_lookup, key=str)
+    [type(None), *sorted(_global_type_lookup, key=str)]
 )
 if sys.version_info[:2] >= (3, 9):
     # subclass of MutableMapping, and in Python 3.9 we resolve to a union
@@ -663,19 +707,18 @@ _global_type_lookup.update(
             # this generates strings that should able to be parsed into integers
             st.from_regex(r"\A-?\d+\Z").filter(functools.partial(can_cast, int)),
         ),
+        typing.SupportsIndex: st.integers() | st.booleans(),
         typing.SupportsBytes: st.one_of(
             st.booleans(),
             st.binary(),
             st.integers(0, 255),
             # As with Reversible, we tuplize this for compatibility with Hashable.
-            st.lists(st.integers(0, 255)).map(tuple),  # type: ignore
+            st.lists(st.integers(0, 255)).map(tuple),
         ),
         typing.BinaryIO: st.builds(io.BytesIO, st.binary()),
         typing.TextIO: st.builds(io.StringIO, st.text()),
     }
 )
-if hasattr(typing, "SupportsIndex"):  # pragma: no branch  # new in Python 3.8
-    _global_type_lookup[typing.SupportsIndex] = st.integers() | st.booleans()
 
 
 # The "extra" lookups define a callable that either resolves to a strategy for
@@ -714,7 +757,7 @@ def register(type_, fallback=None, *, module=typing):
 
         @functools.wraps(func)
         def really_inner(thing):
-            # This branch is for Python < 3.8, when __args__ was not always tracked
+            # This branch is for Python <= 3.8, when __args__ was not always tracked
             if getattr(thing, "__args__", None) is None:
                 return fallback  # pragma: no cover
             return func(thing)
@@ -734,7 +777,7 @@ def register(type_, fallback=None, *, module=typing):
 @register("Type", module=typing_extensions)
 def resolve_Type(thing):
     if getattr(thing, "__args__", None) is None:
-        # This branch is for Python < 3.8, when __args__ was not always tracked
+        # This branch is for Python <= 3.8, when __args__ was not always tracked
         return st.just(type)  # pragma: no cover
     args = (thing.__args__[0],)
     if is_a_union(args[0]):
@@ -758,6 +801,16 @@ def resolve_List(thing):
     return st.lists(st.from_type(thing.__args__[0]))
 
 
+@register(typing.Tuple, st.builds(tuple))
+def resolve_Tuple(thing):
+    elem_types = getattr(thing, "__args__", None) or ()
+    if len(elem_types) == 2 and elem_types[-1] is Ellipsis:
+        return st.lists(st.from_type(elem_types[0])).map(tuple)
+    elif len(elem_types) == 1 and elem_types[0] == ():
+        return st.tuples()  # Empty tuple; see issue #1583
+    return st.tuples(*map(st.from_type, elem_types))
+
+
 def _can_hash(val):
     try:
         hash(val)
@@ -850,9 +903,8 @@ def resolve_ChainMap(thing):
     return resolve_Dict(thing).map(collections.ChainMap)
 
 
-@register("OrderedDict", st.builds(dict).map(collections.OrderedDict))
+@register(typing.OrderedDict, st.builds(dict).map(collections.OrderedDict))
 def resolve_OrderedDict(thing):
-    # typing.OrderedDict is new in Python 3.7.2
     return resolve_Dict(thing).map(collections.OrderedDict)
 
 
@@ -926,7 +978,7 @@ def resolve_Callable(thing):
                 f"Callable type parametrized by {arg!r}.  Consider using an "
                 "explicit strategy, or opening an issue."
             )
-    if getattr(return_type, "__origin__", None) in TypeGuardTypes:
+    if get_origin(return_type) in TypeGuardTypes:
         raise InvalidArgument(
             "Hypothesis cannot yet construct a strategy for callables which "
             f"are PEP-647 TypeGuards (got {return_type!r}).  "
